from operator import attrgetter
import logging
import os
import shutil

import portinus

log = logging.getLogger(__name__)

class ComposeSource(object):

    def __init__(self, name, source=None):
        self.name = name
        self.source = source
        self.path = portinus.get_instance_dir(name)
        self.service_script = self.path.joinpath(name)
        log.debug("Initialized ComposeSource for '{name}' from source: '{source}'".format(name=name, source=source))

    source = property(attrgetter('_source'))

    @source.setter
    def source(self, value):
        if value is not None:
            try:
                with open(os.path.join(value, "docker-compose.yml")):
                    pass
            except FileNotFoundError as e:
                log.error("Unable to access the specified source docker compose file in ({source})".format(source=value))
                raise(e)
        self._source = value

    def _ensure_service_script(self):
        service_script_template = portinus.template_dir.joinpath("service-script")
        shutil.copy(service_script_template, self.service_script)
        os.chmod(self.service_script, 0o755)

    def ensure(self):
        if not self.source:
            log.error("No valid source specified")
            raise(IOError("No valid source specified"))
        log.info("Copying source files for '{self.name}' to '{self.path}'")
        self.remove()
        shutil.copytree(self.source, self.path, symlinks=True, copy_function=shutil.copy)
        self._ensure_service_script()
        log.debug("Successfully copied source files")

    def remove(self):
        log.info("Removing source files for '{name}' from '{path}'".format(name=self.name, path=self.path))
        try:
            shutil.rmtree(self.path)
            log.debug("Successfully removed source files")
        except FileNotFoundError:
            log.debug("No source files found")



#!/usr/bin/env python3

import click
import logging
import sys

import portinus

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.group()
@click.option('-v', '--verbose', count=True, help="Enable more logging. More -v's for more logging")
def task(verbose):
    log_level = logging.WARNING
    if verbose == 1:
        log_level = logging.INFO
    if verbose >= 2:
        log_level = logging.DEBUG
    logging.basicConfig(level=log_level)


@task.command()
@click.option('--name', required=True, help="The name of the service to remove")
def check(name):
    try:
        portinus.monitor.checker.run(name)
    except PermissionError:
        sys.exit(1)


if __name__ == "__main__":
    task()



import logging
from operator import attrgetter

import pathlib
from jinja2 import Template

from .cli import task
from . import restart, monitor
from .environmentfile import EnvironmentFile
from .composesource import ComposeSource
from .service import Service

_script_dir = pathlib.Path(__file__).resolve().parent
template_dir = _script_dir.joinpath("templates")
service_dir = pathlib.Path("/usr/local/portinus-services")


def list():
    """
    List the available services
    """
    _ensure_service_dir()
    print("Available portinus services:")
    for i in sorted(service_dir.iterdir()):
        if i.is_dir():
            print(i.name)


def get_instance_dir(name):
    """
    Get the directory used for storing the service files
    """
    return service_dir.joinpath(name)


def get_template(file_name):
    """
    Returns the named template
    """
    template_file = template_dir.joinpath(file_name)
    with template_file.open() as f:
        template_contents = f.read()

    return Template(template_contents)


def _ensure_service_dir():
    """
    Make sure that the service dir exists
    """
    service_dir.mkdir(exist_ok=True)


class Application(object):
    """
    A portinus Application. This contains all the pieces of a portinus service
    including the restart timer, monitor server, environment file and
    service files themselves
    """

    log = logging.getLogger()

    def __init__(self, name, source=None, environment_file=None, restart_schedule=None):
        self.name = name
        self.environment_file = EnvironmentFile(name, environment_file)
        self.service = Service(name, source)
        self.restart_timer = restart.Timer(name, restart_schedule=restart_schedule)
        self.monitor_service = monitor.Service(name)

    def exists(self):
        return self.service.exists()

    def ensure(self):
        """
        Ensure all the application components are in the correct state
        """
        _ensure_service_dir()
        self.environment_file.ensure()
        self.service.ensure()
        self.restart_timer.ensure()
        self.monitor_service.ensure()

    def remove(self):
        """
        Remove all the application components
        """
        self.service.remove()
        self.environment_file.remove()
        self.restart_timer.remove()
        self.monitor_service.remove()



import logging
from subprocess import check_output

import portinus

import systemd_unit
from . import checker

log = logging.getLogger(__name__)


class Service(object):

    def __init__(self, name):
        self.name = name
        systemd_service_name = portinus.Service(name).service_name
        self._systemd_service = systemd_unit.Unit(systemd_service_name + "-monitor")
        self._systemd_timer = systemd_unit.Unit(systemd_service_name + "-monitor", type="timer")
        pass

    def _generate_service_file(self):
        template = portinus.get_template("monitor.service")
        portinus_monitor_path = check_output(["which", "portinus-monitor"])
        portinus_monitor_path = portinus_monitor_path.decode().strip("\n")

        return template.render(
                name=self.name,
                portinus_monitor_path=portinus_monitor_path
                )

    def _generate_timer_file(self):
        template = portinus.get_template("monitor.timer")

        return template.render(
                name=self.name,
                )

    def ensure(self):
        log.info("Creating/updating {name} monitor timer".format(name=self.name))
        self._systemd_service.ensure(content=self._generate_service_file(), restart=False, enable=False)
        self._systemd_timer.ensure(content=self._generate_timer_file())

    def remove(self):
        log.info("Removing {name} monitor timer".format(name=self.name))
        self._systemd_timer.remove()
        self._systemd_service.remove()



import subprocess
import os
import logging

import systemd_unit

import portinus

log = logging.getLogger(__name__)

class Service(object):

    def __init__(self, name, source=None):
        if not name:
            raise ValueError("Invalid value for 'name'")
        self.name = name
        self.service_name = "{}-{}".format("portinus", name)
        self._source = portinus.ComposeSource(name, source)
        self._systemd_service = systemd_unit.Unit(self.service_name)
        log.debug("Initialized Service for '{name}' with source: '{source}'".format(name=name, source=source))

    def exists(self):
        return os.path.isdir(portinus.get_instance_dir(self.name))

    def _generate_service_file(self):
        start_command = "{service_script} up".format(service_script=self._source.service_script)
        stop_command = "{service_script} down".format(service_script=self._source.service_script)

        template = portinus.get_template("instance.service")
        return template.render(
            name=self.name,
            start_command=start_command,
            stop_command=stop_command,
            )

    def ensure(self):
        log.info("Creating/updating {name} portinus instance".format(name=self.name))
        try:
            self._systemd_service.stop()
        except FileNotFoundError:
            pass
        self._source.ensure()
        self._systemd_service.ensure(content=self._generate_service_file())

    def remove(self):
        log.info("Removing {name} portinus instance".format(name=self.name))
        self._systemd_service.remove()
        self._source.remove()

    def restart(self):
        log.info("Restarting {name}".format(name=self.name))
        self._systemd_service.restart()

    def stop(self):
        log.info("Stopping {name}".format(name=self.name))
        self._systemd_service.stop()

    def compose(self, command):
        log.info("Running compose for {name} with command: '{command}'".format(name=self.name, command=command))
        if not self.exists():
            raise ValueError("The specified service does not exist")
        subprocess.call([self._source.service_script] + list(command))



#!/usr/bin/env python3

import click
import logging
import sys

import portinus

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


def set_log_level(verbose):
    log_level = logging.ERROR
    if verbose == 1:
        log_level = logging.WARNING
    if verbose == 2:
        log_level = logging.INFO
    if verbose >= 3:
        log_level = logging.DEBUG
    logging.basicConfig(level=log_level)


@click.group()
@click.option('-v', '--verbose', count=True, help="Enable more logging. More -v's for more logging")
def task(verbose):
    set_log_level(verbose)


@task.command()
@click.argument('name', required=True)
def remove(name):
    application = portinus.Application(name)
    try:
        application.remove()
    except PermissionError:
        click.echo("Failed to remove the application due to a permissions error")
        sys.exit(1)


@task.command()
@click.argument('name', required=True)
@click.option('--source', type=click.Path(exists=True), required=True, help="A path to a folder containg a docker-compose.yml")
@click.option('--env', help="A file containing the list of environment variables to use")
@click.option('--restart', help="Provide a systemd 'OnCalender' scheduling string to force a restart of the service on the specified interval (e.g. 'weekly' or 'daily')")
def ensure(name, source, env, restart):
    application = portinus.Application(name, source=source,
                                       environment_file=env,
                                       restart_schedule=restart)
    try:
        application.ensure()
    except PermissionError:
        click.echo("Failed to create the application due to a permissions error")
        sys.exit(1)


@task.command()
@click.argument('name', required=True)
def restart(name):
    application = portinus.Application(name)
    try:
        application.service.restart()
    except FileNotFoundError:
        sys.exit(1)


@task.command()
@click.argument('name', required=True)
def stop(name):
    application = portinus.Application(name)
    try:
        application.service.stop()
    except FileNotFoundError:
        click.echo("Unable to find the specified service file")
        sys.exit(1)

@task.command()
@click.argument('name', required=True)
def ps(name):
    application = portinus.Application(name)
    application.service.compose(['ps'])

@task.command()
@click.argument('name', required=True)
def status(name):
    application = portinus.Application(name)
    application.service.compose(['ps'])

@task.command()
def list():
    try:
        portinus.list()
    except FileNotFoundError:
        pass

@task.command()
@click.pass_context
def ls(ctx):
    ctx.forward(list)


@task.command()
@click.argument('name', required=True)
@click.argument('args', required=True, nargs=-1)
def compose(name, args):
    application = portinus.Application(name)
    application.service.compose(args)


if __name__ == "__main__":
    task()



import pathlib
import logging
import os
import shutil

import portinus

log = logging.getLogger(__name__)


class EnvironmentFile(object):

    def __init__(self, name, source_environment_file=None):
        self.name = name
        self._source_environment_file = source_environment_file
        self.path = pathlib.Path("{}.environment".format(portinus.get_instance_dir(self.name)))
        log.debug("Initialized EnvironmentFile for '{name}' from source: '{source_environment_file}'".format(name=name, source_environment_file=source_environment_file))

        if source_environment_file:
            try:
                with open(source_environment_file):
                    pass
            except FileNotFoundError as e:
                log.error("Unable to access the specified environment file ({source_environment_file})".format(source_environment_file=source_environment_file))
                raise(e)

    def __bool__(self):
        return bool(self._source_environment_file)

    def ensure(self):
        if self:
            log.info("Creating/updating environment file for '{name}' at '{path}'".format(name=self.name, path=self.path))
            shutil.copy(self._source_environment_file, self.path)
        else:
            self.remove()

    def remove(self):
        log.info("Removing environment file for {name}".format(name=self.name))
        try:
            os.remove(self.path)
            log.debug("Sucessfully removed environment file")
        except FileNotFoundError:
            log.debug("No environment file found")



#!/usr/bin/env python3
from setuptools import setup, find_packages

setup(
    name="portinus",
    version="1.0.9",
    author="Justin Dray",
    author_email="justin@dray.be",
    url="https://github.com/justin8/portinus",
    description="This utility creates a systemd service file for a docker-compose file",
    packages=find_packages(),
    package_data={'portinus': ['templates/*']},
    license="MIT",
    install_requires=[
        "click",
        "docker",
        "jinja2",
        "systemd_unit"
    ],
    tests_require=[
        "nose",
        "coverage",
        "mock>=2.0.0",
    ],
    test_suite="nose.collector",
    entry_points={
        "console_scripts": [
            "portinus=portinus.cli:task",
            "portinus-monitor=portinus.monitor.cli:task",
        ]
    },
    classifiers=[
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3",
        "Environment :: Console",
        "License :: OSI Approved :: MIT License",
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python",
    ],
)
